from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import redirect, render
from django.utils.crypto import get_random_string
from django.utils.translation import gettext_lazy as _

from allianceauth.services.forms import ServicePasswordForm
from allianceauth.services.hooks import get_extension_logger

from .manager import WikiJSManager

logger = get_extension_logger(__name__)

ACCESS_PERM = 'wikijs.access_wikijs'


@login_required
@permission_required(ACCESS_PERM)
def deactivate_wikijs(request):
    result = WikiJSManager().deactivate_user(request.user)
    if result:
        logger.info("Deactivated Wiki.JS %s" % request.user)
        messages.success(request, _('Deactivated Wiki.JS'))
    else:
        logger.error("Failed to deactivate Wiki.JS for user %s" % request.user)
        messages.error(request, _('An error occurred while processing your Wiki.JS account.'))

    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def activate_wikijs(request):

    credentials = WikiJSManager().activate_user(request.user)

    if credentials:
        logger.info("Activated Wiki.JS %s" % request.user)
        messages.success(request, _('Activated Wiki.JS'))
    else:
        logger.error("Failed to Activate Wiki.JS for user %s" % request.user)
        messages.error(request, _('An error occurred while processing your Wiki.JS account.'))
        return redirect("services:services")

    return render(request, 'services/service_credentials.html', context={'credentials': {"password":credentials}, 'service': 'Wiki.JS'})


@login_required
@permission_required(ACCESS_PERM)
def set_password(request):
    logger.debug("set_wikijs_password called by user %s" % request.user)
    if request.method == 'POST':
        logger.debug("Received POST request with form.")
        form = ServicePasswordForm(request.POST)
        logger.debug("Form is valid: %s" % form.is_valid())
        character = request.user.profile.main_character
        if form.is_valid() and WikiJSManager.user_has_account(request.user) and character is not None:
            password = form.cleaned_data['password']
            logger.debug("Form contains password of length %s" % len(password))
            try:
                result = WikiJSManager()._update_password(request.user.wikijs.uid, password)
                if result:
                    logger.info("Successfully set Wiki.JS password for user %s" % request.user)
                    messages.success(request, _('Set Wiki.JS password.'))
                else:
                    logger.error("Failed to install custom Wiki.JS password for user %s" % request.user)
                    messages.error(request, _('An error occurred while processing your Wiki.JS account.'))
                    return redirect("services:services")
            except Exception:
                logger.error("Failed to install custom Wiki.JS password for user %s" % request.user, exc_info=1)
                messages.error(request, _('An error occurred while processing your Wiki.JS account.'))
            return redirect("services:services")

    else:
        logger.debug("Request is not type POST - providing empty form.")
        form = ServicePasswordForm()

    logger.debug("Rendering form for user %s" % request.user)
    context = {'form': form, 'service': 'Wiki.JS'}
    return render(request, 'services/service_password.html', context=context)


@login_required
@permission_required(ACCESS_PERM)
def reset_password(request):
    password = get_random_string(15)
    result = WikiJSManager()._update_password(request.user.wikijs.uid, password)
    if result:
        logger.info("Successfully set Wiki.JS password for user %s" % request.user)
        messages.success(request, _('Set Wiki.JS password.'))
    else:
        logger.error("Failed to install custom Wiki.JS password for user %s" % request.user)
        messages.error(request, _('An error occurred while processing your Wiki.JS account.'))
        return redirect("services:services")

    return render(request, 'services/service_credentials.html', context={'credentials': {"password":password}, 'service': 'Wiki.JS'})



from django.conf import settings

WIKIJS_AADISCORDBOT_INTEGRATION = getattr(settings, 'WIKIJS_AADISCORDBOT_INTEGRATION', True)

WIKIJS_API_URL = getattr(settings, 'WIKIJS_API_URL', settings.WIKIJS_URL)



# Generated by Django 2.2.12 on 2020-06-15 09:32

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wikijs', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='WikiJsUser',
            new_name='WikiJs',
        ),
    ]



from unittest import mock

from django.conf import settings
from django.contrib.auth.models import Group, Permission, User
from django.core.exceptions import ObjectDoesNotExist
from django.test import RequestFactory, TestCase

from allianceauth.tests.auth_utils import AuthUtils

from .auth_hooks import WikiJSService
from .models import WikiJs

MODULE_PATH = 'wikijs'
DEFAULT_AUTH_GROUP = 'Member'


def add_permissions():
    permission = Permission.objects.get(codename='access_wikijs')
    members = Group.objects.get_or_create(name=DEFAULT_AUTH_GROUP)[0]
    AuthUtils.add_permissions_to_groups([permission], [members])


class WikiJSHooksTestCase(TestCase):
    def setUp(self):
        self.member = 'member_user'
        member = AuthUtils.create_member(self.member)
        WikiJs.objects.create(user=member, uid=3)
        self.none_user = 'none_user'
        self.none_user = AuthUtils.create_user(self.none_user)
        self.service = WikiJSService
        self.del_user = 'del_user'
        self.del_user = AuthUtils.create_user(self.del_user)
        add_permissions()

    def test_has_account(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)
        self.assertTrue(service.user_has_account(member))
        self.assertFalse(service.user_has_account(none_user))

    def test_service_enabled(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)

        self.assertTrue(service.service_active_for_user(member))
        self.assertFalse(service.service_active_for_user(none_user))

    def test_delete_user_with_no_wiki(self):  # this doesn't fail properly on sqlite investigate more tests on mysql/psql
        User.objects.get(username=self.del_user).delete()

    @mock.patch(MODULE_PATH + '.manager.WikiJSManager._update_user')
    def test_update_user(self, disable):
        disable.execute.return_value = True
        service = self.service()
        # Test member is not deleted
        member = User.objects.get(username=self.member)
        self.assertTrue(service.update_groups(member))
        self.assertTrue(disable.called)

    @mock.patch(MODULE_PATH + '.manager.WikiJSManager._update_user')
    def test_update_non_user(self, disable):
        disable.execute.return_value = True
        service = self.service()
        # Test member is not deleted
        member = User.objects.get(username=self.none_user)
        self.assertFalse(service.update_groups(member))
        self.assertFalse(disable.called)

    @mock.patch(MODULE_PATH + '.manager.WikiJSManager._update_user')
    def test_update_all_users(self, disable):
        disable.execute.return_value = True
        service = self.service()
        # Test member is not deleted
        service.update_all_groups()
        self.assertEqual(disable.call_count, 1)

    @mock.patch(MODULE_PATH + '.manager.WikiJSManager.client')
    def test_validate_user(self, disable):
        disable.execute.return_value = '{"data": {"users": {"deactivate": {"responseResult": {"succeeded": true}}}}}'
        service = self.service()
        # Test member is not deleted
        member = User.objects.get(username=self.member)
        service.validate_user(member)
        self.assertTrue(member.wikijs)

        # Test none user is deleted
        none_user = User.objects.get(username=self.none_user)
        WikiJs.objects.create(user=none_user, uid=4)
        service.validate_user(none_user)
        with self.assertRaises(ObjectDoesNotExist):
            User.objects.get(username=self.none_user).wikijs

    @mock.patch(MODULE_PATH + '.manager.WikiJSManager.client')
    def test_delete_user(self, disable):
        disable.execute.return_value = '{"data": {"users": {"deactivate": {"responseResult": {"succeeded": true}}}}}'
        member = User.objects.get(username=self.member)
        service = self.service()
        result = service.delete_user(member)

        self.assertTrue(result)
        with self.assertRaises(ObjectDoesNotExist):
            User.objects.get(username=self.member).wikijs

    def test_render_services_ctrl(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        request = RequestFactory().get('/services/')
        request.user = member

        response = service.render_services_ctrl(request)
        self.assertTemplateUsed(service.service_ctrl_template)
        self.assertIn('href="%s"' % settings.WIKIJS_URL, response)



# Cog used by https://github.com/pvyParts/allianceauth-discordbot
import logging

from aadiscordbot.app_settings import get_all_servers
from aadiscordbot.cogs.utils.decorators import sender_has_perm
from discord.commands import SlashCommandGroup
from discord.embeds import Embed
from discord.ext import commands

from django.conf import settings

from wikijs.manager import WikiJSManager

logger = logging.getLogger('aadiscordbot.cogs.wikijs')


class Wikijs(commands.Cog):
    """
    WikiJS relevant cogs for AADiscordbot
    Currently only implements minimal search
    """

    def __init__(self, bot):
        self.bot = bot

    wikijs_commands = SlashCommandGroup(
        "wiki",
        "Wiki JS",
        guild_ids=get_all_servers())

    @wikijs_commands.command(name="search", description="Search WikiJS")
    @sender_has_perm('wikijs.access_wikijs')
    async def search(self, ctx, search_string: str):
        """
        Returns the top Wiki Article search result for a string
        """
        await ctx.respond(content=f"Searching for {search_string}", ephemeral=True)
        try:
            pagesearchresponse = WikiJSManager().search_for_page(search_string)
        except Exception as e:
            logger.exception(e)

        embed = Embed(title=f"WikiJS Search: {search_string}")

        for result in pagesearchresponse["data"]["pages"]["search"]["results"]:
            title = result["title"]
            path = result["path"]
            embed.add_field(
                name=title,
                value=f"{settings.WIKIJS_URL}{path}"
            )
        return await ctx.respond(embed=embed)

    @wikijs_commands.command(name="link", description="Link to the Wiki")
    async def link(self, ctx):
        await ctx.respond(f"{settings.WIKIJS_URL}")


def setup(bot):
    bot.add_cog(Wikijs(bot))
